#pragma semicolon 1

#include <sourcemod>
#include <sdktools>
#include <smartdm>
#include <shop>

// Zombie:Reloaded support
#tryinclude <zombiereloaded>

#define PLUGIN_VERSION	"1.7"

enum Trail
{
	String:Name[64],
	String:Color[32],
    String:Material[256],
	bool:AbleToSell,
    StartingWidth,
    Price
}

#define CATEGORY	"trails"

new Trail:g_trails[200][Trail];
new g_trailCount = 0;
new g_SpriteModel[MAXPLAYERS + 1] = {-1, ...};

new Handle:g_hLifeTime;

public Plugin:myinfo =
{
	name = "[Shop] Trails",
	author = "FrozDark (HLModders LLC)",
	description = "Additional plugin to shop - Trails",
	version = PLUGIN_VERSION,
	url = "http://www.hlmod.ru/"
};

public OnPluginStart()
{
	g_hLifeTime = CreateConVar("sm_shop_trails_lifetime", "1.0", "Lifetime.", 0, true, 0.1);
	
	HookEvent("player_spawn", PlayerSpawn);
	HookEvent("player_death", PlayerDeath);
	HookEvent("player_team", PlayerTeam);
	
	if (Shop_IsConnected()) Shop_OnConnected();
}

public Shop_OnConnected()
{
	Shop_RegisterCategory(CATEGORY, "Трейлы", OnCategoryRegistered);
}

public OnCategoryRegistered(const String:category[], const String:name[])
{
	g_trailCount = 0;
	
	decl String:_buffer[PLATFORM_MAX_PATH];
	GetCfgFile(_buffer, sizeof(_buffer), "trails.txt");
	
	new Handle:kv = CreateKeyValues("Trails");
	
	if (!FileToKeyValues(kv, _buffer)) SetFailState("\"%s\" not found", _buffer);
	
	if (KvGotoFirstSubKey(kv))
	{
		do
		{
			KvGetString(kv, "material", g_trails[g_trailCount][Material], 256);
			if (!File_ExtEqual(g_trails[g_trailCount][Material], "vmt")) continue;
			
			KvGetSectionName(kv, g_trails[g_trailCount][Name], 64);

			KvGetString(kv, "color", g_trails[g_trailCount][Color], 256);
            
			g_trails[g_trailCount][Price] = KvGetNum(kv, "price");
			g_trails[g_trailCount][AbleToSell] = bool:KvGetNum(kv, "cansell"); 
			g_trails[g_trailCount][StartingWidth] = KvGetNum(kv, "startingwidth");
			
			Downloader_AddFileToDownloadsTable(g_trails[g_trailCount][Material]);
			PrecacheModel(g_trails[g_trailCount][Material]);
			
			Shop_AddItemToCategory(CATEGORY, g_trails[g_trailCount][Name], g_trails[g_trailCount][Price], Item_Togglable, g_trails[g_trailCount][AbleToSell], g_trails[g_trailCount][Price]/2, _, _, OnEquipItem);
			
			g_trailCount++;
		} while (KvGotoNextKey(kv));
	}
	
	CloseHandle(kv);
}

public OnMapStart()
{
	for (new i = 0; i < g_trailCount; i++)
	{
		Downloader_AddFileToDownloadsTable(g_trails[i][Material]);
		PrecacheModel(g_trails[i][Material], true);
	}
}

public OnPluginEnd()
{
	Shop_UnregisterMe();
	for (new i = 1; i <= MaxClients; i++)
	{
		KillTrail(i);
	}
}

new selected_id[MAXPLAYERS+1] = {-1, ...};
public ShopAction:OnEquipItem(client, const String:category[], const String:item[], itemID, bool:toggledOn)
{
	if (toggledOn)
	{
		selected_id[client] = -1;
		OnClientDisconnect(client);
		
		return Shop_ToggleOff;
	}
	
	Shop_ToggleCategoryOff(client, category);
	
	for (new i = 0; i < g_trailCount; i++)
	{
		if (StrEqual(item, g_trails[i][Name], false))
		{
			selected_id[client] = i;
	
			SpriteTrail(client);
			
			return Shop_ToggleOn;
		}
	}
	
	return Shop_Raw;
}

public OnMapEnd()
{
	for (new client = 1; client <= MAXPLAYERS; client++)
	{
		g_SpriteModel[client] = -1;
	}
}


public OnClientDisconnect(client)
{
	KillTrail(client);
	
	selected_id[client] = -1;
	g_SpriteModel[client] = -1;
}

public PlayerSpawn(Handle:event,const String:name[],bool:dontBroadcast)
{
	new client = GetClientOfUserId(GetEventInt(event, "userid"));
	
	CreateTimer(1.0, GiveTrail, client);
}

public PlayerTeam(Handle:Spawn_Event, const String:Death_Name[], bool:Death_Broadcast )
{
	if (GetEventInt(Spawn_Event, "team") <= 1)
	{
		KillTrail(GetClientOfUserId(GetEventInt(Spawn_Event, "userid")));
	}
}

public PlayerDeath(Handle:event,const String:name[],bool:dontBroadcast)
{
	new client = GetClientOfUserId(GetEventInt(event, "userid"));

	KillTrail(client);
}

public Action:GiveTrail(Handle:timer, any:client)
{
	SpriteTrail(client);
}

#if defined _zr_included
public ZR_OnClientInfected(client, attacker, bool:motherInfect, bool:respawnOverride, bool:respawn)
{
	SpriteTrail(client);
}
#endif

SpriteTrail(client)
{
	if (!client)
	{
		return;
	}

	KillTrail(client);
	
	if (selected_id[client] == -1 || !IsClientInGame(client) || IsFakeClient(client) || !IsPlayerAlive(client) || !(1 < GetClientTeam(client) < 4))
	{
		return;
	}
	
	g_SpriteModel[client] = CreateEntityByName("env_spritetrail");
	if (g_SpriteModel[client] != -1) 
	{
		DispatchKeyValueFloat(g_SpriteModel[client], "lifetime", GetConVarFloat(g_hLifeTime));
		DispatchKeyValue(g_SpriteModel[client], "endwidth", "6.0");
		DispatchKeyValue(g_SpriteModel[client], "startwidth","10");
		DispatchKeyValue(g_SpriteModel[client], "spritename", g_trails[selected_id[client]][Material]);
		DispatchKeyValue(g_SpriteModel[client], "renderamt", "255");
		DispatchKeyValue(g_SpriteModel[client], "rendercolor", g_trails[selected_id[client]][Color]);
		DispatchKeyValue(g_SpriteModel[client], "rendermode", "1");
		
		DispatchSpawn(g_SpriteModel[client]);
		
		decl Float:Client_Origin[3];
		GetClientAbsOrigin(client,Client_Origin);
		Client_Origin[2] += 10.0; //Beam clips into the floor without this
		
		TeleportEntity(g_SpriteModel[client], Client_Origin, NULL_VECTOR, NULL_VECTOR);
		
		SetVariantString("!activator");
		AcceptEntityInput(g_SpriteModel[client], "SetParent", client); 
		SetEntPropFloat(g_SpriteModel[client], Prop_Send, "m_flTextureRes", 0.05);
	}
}

KillTrail(client)
{
	if (g_SpriteModel[client] > MaxClients && IsValidEdict(g_SpriteModel[client]))
	{
		AcceptEntityInput(g_SpriteModel[client], "kill");
	}
	
	g_SpriteModel[client] = -1;
}

stock File_GetExtension(const String:path[], String:buffer[], size)
{
	new extpos = FindCharInString(path, '.', true);
	
	if (extpos == -1)
	{
		buffer[0] = '\0';
		return;
	}

	strcopy(buffer, size, path[++extpos]);
}

stock File_ExtEqual(const String:path[], const String:ext[], bool:caseSensetive = false)
{
	decl String:buf[4];
	File_GetExtension(path, buf, sizeof(buf));
	return StrEqual(buf, ext, caseSensetive);
}